<div id="sidebar-header" class="menu-box">
    <div class="menu-title">
        <h2><a href="https://api.kde.org">api.kde.org</a></h2>
    </div>
</div>
<div class="menu-box">
    <div class="menu-content">
        <p>If you are looking for documentation about KDE software or how to use Plasma, please refer to our <a href="https://userbase.kde.org">userbase website</a>.</p>
    </div>
</div>
<div class="menu-box">
    <div class="menu-title">
        <h2>Other KDE Products</h2>
    </div>
    <div class="menu-content">
    <ul>
        <li><a href="https://api.kde.org/other.php">Other KDE Software</a></li>
        <li><a href="https://api.kde.org/ecm/">Extra CMake Modules</a></li>
    </ul>
    </div>
</div>
<div class="menu-box">
    <div class="menu-title">
    <h2>Deprecated Products</h2>
    </div>
    <div class="menu-content">
    <ul>
        <li><a href="https://api.kde.org/cmake/modules.html">KDE CMake Modules</a></li>
        <li><a href="https://api.kde.org/history4.php">KDE4 Versions</a></li>
        <li><a href="https://api.kde.org/history.php">KDE3 and older versions</a></li>
    <ul>
    </div>
</div>
